@@ -1,5 +1,7 @@
 #![cfg_attr(docsrs, feature(doc_cfg))]
 #![no_std]
+#![cfg_attr(target_arch = "avr", feature(llvm_asm))]
+#![cfg_attr(target_arch = "avr", feature(extended_key_value_attributes))]
 #![doc = include_str!("../README.md")]
 
 pub use bare_metal::CriticalSection;
@@ -118,6 +120,25 @@ cfg_if::cfg_if! {
                 cortex_m::interrupt::enable()
             }
         }
+    } else if #[cfg(target_arch = "avr")] {
+        #[no_mangle]
+        unsafe fn _critical_section_acquire() -> u8 {
+            let mut sreg: u8;
+            llvm_asm!(
+                "in $0, 0x3F
+                 cli"
+                : "=r"(sreg)
+                ::: "volatile"
+            );
+            sreg
+        }
+
+        #[no_mangle]
+        unsafe fn _critical_section_release(token: u8) {
+            if token & 0x80 == 0x80 {
+                llvm_asm!("sei" :::: "volatile");
+            }
+        }
     } else if #[cfg(target_arch = "riscv32")] {
         #[no_mangle]
         unsafe fn _critical_section_acquire() -> u8 {
